///////////////////////////////////////////////////
// Calculating the total aquisition number (weighted version, Poisson distribution)
// Author: Pawel Pralat
// Last modification: November 19, 2015
///////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// n : the length of the path
const int n = 100000000;

// the max weight generated is (aas) ln(n)/ln(ln(n)) so let's keep 100 as max value (which should be much bigger)
const int k = 100;

// table containing weights
int t[n];

// Poisson distribution p[i] is the probability Po(1) \le i
double p[n]; 

// Testing if interval [a, b] can be collapsed to one vertex
int test(int a, int b)
{
  int l = a;
  int tmp = t[a];
  while ( (l <= b) && (tmp <= t[l+1]) )
    {
      tmp += t[l+1];
      l++;
    }

  int r = b;
  tmp = t[b];
  while ( (r >= a) && (tmp <= t[r-1]) )
    {
      tmp += t[r-1];
      r--;
    }

  if ( r-l <= 1 )
    return 1;
  else 
    return 0;
}

///////////////////////////////////////////////////
// Main program
///////////////////////////////////////////////////
int main(int argc, char * argv[])
{
  printf("n = %d\n", n);

  // Generating Poisson distribution
  p[0] = 1/exp(1.0);
  for (int i = 1 ; i < k ; i++)
    p[i] = p[i-1]/(1.0*i);
  // Cumulative distribution 
  for (int i = k-2 ; i >= 0 ; i--)
    p[i] = p[i] + p[i+1];
  p[0] = 2.0; // just to make sure there is no problem with rounding ;-)

  long int nr = 0;

  for (int iter = 0 ; iter < 1000 ; iter++)
    {

  // generating weights
  for (int i = 0 ; i < n ; i++)
    {
      double x = 1.0 * rand() / RAND_MAX;
      int j = k-1;
      while (x > p[j]) j--;
      t[i] = j;
      // printf("%d ", j);
    }
  // printf("\n");

  int i = 0;

  while (i < n)
    {
      int j = 0;
      while ( (test(i,i+j)) && (i+j < n) )
	j++;

      //      for (int a = i ; a < i+j ; a++)
      //printf("%d ",t[a]);
      //printf("| ");
      
      i += j;
      nr++;
    }

  printf("iterations = %3d, %ld = %22.20f n\n", iter+1, nr, 1.0*nr/(1.0*(iter+1.0)*((double) n)));
    }
  return 0;
}
